#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from inclusivewriting.file_utils import read_file

class FileUtilsTestSuite(unittest.TestCase):
    def setUp(self):
        pass

    def test_read_file(self):
        text = read_file(__file__) # Get current filename
        # The length must be greater than 0 
        self.assertTrue(len(text) > 0)

if __name__ == '__main__':
    unittest.main()



#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from inclusivewriting.unicode_utils import get_all_punctuation_separator_characters

class UnicodeUtilsTestSuite(unittest.TestCase):
    def setUp(self):
        pass

    def test_all_punctuation_separator_characters(self):
        punctuations_separator = get_all_punctuation_separator_characters()
        # Ensure that we have more than zero separator/punctuation characters
        self.assertTrue(len(punctuations_separator) > 0)

if __name__ == '__main__':
    unittest.main()



#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from tests.unicode_utils_test import *
from tests.file_utils_test import *
from tests.locale_utils_test import *
from tests.suggestions_test import *

if __name__ == '__main__':
    unicode_utils_tests = UnicodeUtilsTestSuite()
    file_utils_tests = FileUtilsTestSuite()
    suggestions_tests = SuggestionsTestSuite()
    locales_tests = LocaleUtilsTestSuite()
    tests = unittest.TestSuite([unicode_utils_tests, 
                                file_utils_tests,
                                suggestions_tests,
                                locales_tests])
    unittest.main()



#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

def read_input_from_terminal():
    text = None
    try:
        text = input()
    except EOFError:
        text = ""
        pass

    return text




#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""inclusivewriting is a multilingual application for inclusive writing. Languages evolve. Developed in Python, this application helps to write texts in an inclusive way. The application detects possibly problematic phrases and suggests some replacements."""

__version__ = "0.3"



#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import json
import pkg_resources


import typer
import re
from rich import print
from inclusivewriting.file_utils import read_file
from inclusivewriting.unicode_utils import get_all_punctuation_separator_characters
from inclusivewriting.locale_utils import get_default_locale_encoding, get_default_locale_message_handler

def get_all_language_resource_config_file(config_file:str=None):
    # File containing links to resources
    if config_file is None or config_file == "":
       all_language_resource_file = pkg_resources.resource_filename('inclusivewriting', 'configuration.json')
       return all_language_resource_file
    return config_file

class Lexeme:
    def __init__(self, value, links=list()):
        self.value = value
        self.links = links

    def get_value(self):
        return self.value

    def get_links(self):
        return self.links

    def __str__(self):
        return '"' + self.get_value() + '" : ' + "[ " + ", ".join('"{0}"'.format(link) for link in self.get_links())  + " ]"

class Replacement:
    def __init__(self, lexeme, references=list()):
        self.lexeme = lexeme
        self.references = references

    def get_value(self):
        return self.lexeme.get_value()

    def __str__(self):
        return '"' + self.get_value() + '" : ' + '{ "links" : [' + ", ".join('"{0}"'.format(link) for link in self.lexeme.get_links()) + '], "references" : [' + ", ".join('"{0}"'.format(reference) for reference in self.references)  + " ] " "}"

class Suggestion:
    def __init__(self, lexeme, replacement_lexemes):
        self.lexeme = lexeme
        self.replacement_lexemes = replacement_lexemes

    def get_replacement_lexemes(self):
        return self.replacement_lexemes

    def get_lexeme(self):
        return self.lexeme

    def __str__(self):
        return "{ " + '"' + str(self.get_lexeme().get_value()) + '" : {'  + ", ".join(str(replacement) for replacement in self.get_replacement_lexemes())  + "}" "}" 


def get_all_language_resources(config_file:str=None):
    """
         This method can be used to obtain all the language resources configured in the configuration 
         file. If the configuration file is not specified, the default configuration file is used

        Parameters
        ----------
          config_file : str
            This parameter contains the file path of a congiguration file

        Returns
        -------
          resources
            language resources

    """
    config_file = get_all_language_resource_config_file(config_file)
    resources = dict()
    resource_file_string = read_file(config_file)
    resources_parsed_data = json.loads(resource_file_string)
    for key, value in resources_parsed_data.items():
        resources[key] = value
    return resources


def get_suggestions(language: str, config_file:str=None):
    """
         This method can be used to obtain the suggestions for a given language as
         configured in the configuration 
         file. If the configuration file is not specified, the default configuration file is used

        Parameters
        ----------
          language : str
            This parameter is the language code or locale (en: English)
          config_file : str
            This parameter contains the file path of a congiguration file

        Returns
        -------
          suggestions
            suggestions for a given language

    """
    config_file = get_all_language_resource_config_file(config_file)
    resources = get_all_language_resources(config_file)
    suggestions = dict()

    # Load all the suggestion files for a given language
    if language in resources:
        for suggestion_file in resources[language]:
            suggestion_file = pkg_resources.resource_filename('inclusivewriting', suggestion_file)
            suggestion_file_data = read_file(suggestion_file)
            suggestion = json.loads(suggestion_file_data)
            for key, value in suggestion.items():
                lexeme =  Lexeme(key, value["lexeme"])
                replacements = list()
                for replacement in value["replacement"]:
                    replacement_lexeme = Lexeme(replacement, value["replacement"][replacement]["lexeme"])
                    replacement = Replacement(replacement_lexeme, value["replacement"][replacement]["references"])
                    replacements.append(replacement)

                suggestion_value = Suggestion(lexeme, replacements)
                suggestions[key] = suggestion_value

    return suggestions

def detect_and_get_suggestions(text, config_file:str=None):
    """
         This method detects the language/locale and 
         returns the suggestions for the language and the given text.
          If the configuration file is not specified, the default configuration file is used

        Parameters
        ----------
          text : str
            The text for which the suggestions are required
          config_file : str
            This parameter contains the file path of a congiguration file

        Returns
        -------
          suggestions
            suggestions for the current locale and the given text

    """
    config_file = get_all_language_resource_config_file(config_file)
    language, encoding = get_default_locale_encoding()
    punctuations_separator = get_all_punctuation_separator_characters()
    suggestions = get_suggestions(language, config_file)

    punctuations_separator = "|".join(punctuations_separator)
    words = re.split("(" + punctuations_separator + ")", text)
    used_suggestions = set()
    
    updated_text = ""
    for word in words:
        if word.lower() in suggestions:
            updated_text = updated_text + "<change>" + word + "</change>"
            used_suggestions.add(word)
        else:
            updated_text = updated_text +  word 

    return used_suggestions, suggestions, updated_text



#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import sys
import re
import unicodedata

def get_all_punctuation_separator_characters():
    characters = range(sys.maxunicode)
    # Get all punctuation and characters, i.e., unicode category
    # belonging to P or Z
    # Escaping the characters ensures that these can be later used
    # splitting text
    punctuations_separator = set(re.escape(chr(i)) for i in characters
                             if unicodedata.category(chr(i)).startswith("P") or
                                unicodedata.category(chr(i)).startswith("Z"))
    return(punctuations_separator)




import locale
import pkg_resources
import gettext

def get_default_locale_encoding():
    default, encoding = locale.getdefaultlocale()
    return default, encoding

def get_default_locale_message_handler():
    default, encoding = get_default_locale_encoding()
    localesdir = pkg_resources.resource_filename('inclusivewriting', 'locales')
    lang = gettext.translation('inclusivewriting', localedir=localesdir, languages=[default])
    lang.install()
    return lang.gettext



#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import json

import typer
import re
from rich import print
from inclusivewriting.file_utils import read_file
from inclusivewriting.text_utils import read_input_from_terminal
from inclusivewriting.locale_utils import get_default_locale_message_handler
from inclusivewriting.suggestions import detect_and_get_suggestions

app = typer.Typer()

@app.command()
def detect(language: str,
           config: str = typer.Option(None,
               help="Use a different configuration file"),
           filepath: str = typer.Option(None, 
                  help="File name; if missing, you will be prompted to enter a text")):
    _ = get_default_locale_message_handler()
    text = None
    if filepath is None:
        print(_("Enter [bold magenta]a text[/bold magenta]."), end="")
        print(_(" Press [bold magenta]Ctrl+D[/bold magenta] to exit:"))
        text = read_input_from_terminal()
    else:
        text = read_file(filepath)
    used_suggestions, suggestions, updated_text = detect_and_get_suggestions(text, config)
    updated_text = updated_text.replace("<change>", "[bold green]")
    updated_text = updated_text.replace("</change>", "[/bold green]")
    print(updated_text)
    print()
    if (len(used_suggestions) > 0):
        print("Following are some suggested replacements:")
        for word in used_suggestions:
            print("[bold green]" + word + " [/bold green]: ", end="")
            for replacement_lexeme in suggestions[word.lower()].get_replacement_lexemes():
                print("[bold blue]" + replacement_lexeme.get_value() + "[/bold blue]", end="; ")
            print()
    print()

if __name__ == "__main__":
    app()



#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

def read_file(filename):
    text = None
    with open(filename, "r") as text_file:
        text = text_file.read()
    return text




#
# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from inclusivewriting.locale_utils import *

class LocaleUtilsTestSuite(unittest.TestCase):
    def setUp(self):
        pass

    def test_get_default_locale_message_handler(self):
        _ = get_default_locale_message_handler() # get message handler
        # This must return a message handler 
        self.assertTrue(_ != None)

    def test_get_default_locale_encoding(self):
        language, encoding = get_default_locale_encoding() # get locale and encoding
        # These values must not be blank
        self.assertTrue(language != None)
        self.assertTrue(encoding != None)

if __name__ == '__main__':
    unittest.main()



#
# SPDX-FileCopyrightText: 2020 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import setuptools
import runpy

version_meta = runpy.run_path("./inclusivewriting/version.py")
VERSION = version_meta["__version__"]

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="inclusivewriting",
    version=VERSION,
    author="John Samuel",
    author_email="johnsamuelwrites@example.com",
    description="Python multilingual application for inclusive writing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/johnsamuelwrites/inclusive",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_dir={'inclusivewriting': 'inclusivewriting'},
    package_data = {"inclusivewriting": ["configuration.json", "resources/*/*.json", 
        "locales/*", "locales/*/*", "locales/*/*/*"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License" +
        " v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'rich>=10.9.0',
        'typer>=0.3.2',
        'babel>=2.9.1'
    ],
    python_requires='>=3.7',
)



#

# SPDX-FileCopyrightText: 2022 John Samuel <johnsamuelwrites@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from inclusivewriting.suggestions import *

class SuggestionsTestSuite(unittest.TestCase):
    def setUp(self):
        self.config_file = "./inclusivewriting/configuration.json"
        pass

    def test_get_all_language_resources(self):
        resources = get_all_language_resources(self.config_file) # Get resources for all languages
        # There must be at least one language resource
        self.assertTrue(len(resources) > 0)

        # A language resource must not be empty
        for language in resources.keys():
            self.assertTrue(len(resources[language]) > 0)

    def test_en_language_resource(self):
        suggestions = get_suggestions("en", self.config_file) # Get suggestions for "en"
        # There must be at least one suggestion 
        self.assertTrue(len(suggestions) > 0)

    def test_detect_get_suggestions(self):
        text = "Once the user has installed the packages, he can run the application"
        used_suggestions, suggestions, updated_text = detect_and_get_suggestions(text, self.config_file)
        self.assertTrue(updated_text=="Once the user has installed the packages, <change>he</change> can run the application")
        self.assertTrue("he" in used_suggestions)

    def test_get_all_language_resources_without_config(self):
        resources = get_all_language_resources("") # Get resources for all languages and empty configuration file
        # There must be at least one language resource
        self.assertTrue(len(resources) > 0)

        # A language resource must not be empty
        for language in resources.keys():
            self.assertTrue(len(resources[language]) > 0)

    def test_en_language_resource_without_config(self):
        suggestions = get_suggestions("en", "") # Get suggestions for "en" and empty configuration file
        # There must be at least one suggestion 
        self.assertTrue(len(suggestions) > 0)

    def test_detect_get_suggestions_without_config(self):
        text = "Once the user has installed the packages, he can run the application"
        used_suggestions, suggestions, updated_text = detect_and_get_suggestions(text, None) # No configuration file
        self.assertTrue(updated_text=="Once the user has installed the packages, <change>he</change> can run the application")
        self.assertTrue("he" in used_suggestions)
    def test_class_Lexeme(self):
        lexeme = Lexeme("he", ["https://www.wikidata.org/wiki/Lexeme:L485"])
        self.assertEqual(str(lexeme), '"he" : [ "https://www.wikidata.org/wiki/Lexeme:L485" ]')

    def test_class_Replacement(self):
        replacement_lexeme = Lexeme("they", ["https://www.wikidata.org/wiki/Lexeme:L371"])
        replacement = Replacement(replacement_lexeme,
                   ["https://en.wikipedia.org/wiki/Inclusive_language"])
        self.assertEqual(str(replacement), '"they" : { "links" : ["https://www.wikidata.org/wiki/Lexeme:L371"], "references" : ["https://en.wikipedia.org/wiki/Inclusive_language" ] }')

    def test_class_Suggestion(self):
        lexeme = Lexeme("he", ["https://www.wikidata.org/wiki/Lexeme:L485"])
        replacement_lexeme = Lexeme("they", ["https://www.wikidata.org/wiki/Lexeme:L371"])
        replacement = Replacement(replacement_lexeme,
                   ["https://en.wikipedia.org/wiki/Inclusive_language"])
        suggestion = Suggestion(lexeme, [replacement])
        self.assertEqual(str(suggestion), '{ "he" : {"they" : { "links" : ["https://www.wikidata.org/wiki/Lexeme:L371"], "references" : ["https://en.wikipedia.org/wiki/Inclusive_language" ] }}}')

if __name__ == '__main__':
    unittest.main()
